# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
        ('latest_tweets', '0006_verbose_names'),
    ]

    operations = [
        migrations.AddField(
            model_name='tweet',
            name='html',
            field=models.TextField(default='', verbose_name='HTML'),
            preserve_default=False,
        ),
    ]



from django.conf import settings
from django.core.management.base import BaseCommand
from django.db import transaction
from twitter import OAuth, Twitter

from latest_tweets.models import Tweet
from latest_tweets.utils import update_tweets


@transaction.atomic
def update_user(user):
    t = Twitter(auth=OAuth(
        settings.TWITTER_OAUTH_TOKEN,
        settings.TWITTER_OAUTH_SECRET,
        settings.TWITTER_CONSUMER_KEY,
        settings.TWITTER_CONSUMER_SECRET
    ))
    messages = t.statuses.user_timeline(screen_name=user, include_rts=True)
    tweet_list = update_tweets(messages=messages)

    # To ensure we delete any deleted tweets
    oldest_date = None
    tweet_id_list = []

    for i in tweet_list:
        # Help prune out deleted tweets
        if not oldest_date or i.created < oldest_date:
            oldest_date = i.created

        tweet_id_list.append(i.id)

    # Remove any deleted tweets in our date range
    Tweet.objects.filter(user=user, created__gt=oldest_date).exclude(id__in=tweet_id_list).delete()


class Command(BaseCommand):
    args = 'user [user ...]'

    def handle(self, *args, **options):
        for i in args:
            update_user(i)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
        ('latest_tweets', '0003_tweet_is_reply'),
    ]

    operations = [
        migrations.AlterField(
            model_name='tweet',
            name='user',
            field=models.CharField(db_index=True, max_length=15),
            preserve_default=True,
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
        ('latest_tweets', '0009_counts'),
    ]

    operations = [
        migrations.AlterField(
            model_name='photo',
            name='photo_id',
            field=models.BigIntegerField(verbose_name='Photo ID'),
        ),
        migrations.AlterUniqueTogether(
            name='photo',
            unique_together=set([('tweet', 'photo_id')]),
        ),
    ]



from django.core.management.base import BaseCommand
from django.conf import settings
from twitter.oauth_dance import oauth_dance


def get_auth_tokens(stdout):
    oauth_token, oauth_secret = oauth_dance(
        'django-latest-tweets', settings.TWITTER_CONSUMER_KEY, settings.TWITTER_CONSUMER_SECRET)

    stdout.write("\nNow add the following lines to your settings.py:\n\n")
    stdout.write("TWITTER_OAUTH_TOKEN = '%s'\n" % (oauth_token,))
    stdout.write("TWITTER_OAUTH_SECRET = '%s'\n\n" % (oauth_secret,))


class Command(BaseCommand):
    def handle(self, *args, **options):
        get_auth_tokens(self.stdout)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
        ('latest_tweets', '0004_username_length'),
    ]

    operations = [
        migrations.AddField(
            model_name='tweet',
            name='name',
            field=models.CharField(default='', max_length=20),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name='tweet',
            name='retweeted_name',
            field=models.CharField(max_length=20, blank=True),
            preserve_default=True,
        ),
    ]



from __future__ import unicode_literals

from django.db import models
from django.utils.encoding import python_2_unicode_compatible


@python_2_unicode_compatible
class Tweet(models.Model):
    tweet_id = models.BigIntegerField('Tweet ID', unique=True)
    user = models.CharField(max_length=15, db_index=True)
    name = models.CharField(max_length=20)
    text = models.CharField(max_length=250)
    html = models.TextField('HTML')
    favorite_count = models.PositiveIntegerField(default=0)
    retweet_count = models.PositiveIntegerField(default=0)
    retweeted_username = models.CharField(max_length=20, blank=True)
    retweeted_name = models.CharField(max_length=20, blank=True)
    retweeted_tweet_id = models.BigIntegerField('Retweeted tweet ID', null=True, blank=True)
    is_reply = models.BooleanField(default=False, db_index=True)
    created = models.DateTimeField(db_index=True)

    class Meta:
        ordering = ('-created',)

    def __str__(self):
        return '@%s - %s' % (self.user, self.text)

    def get_absolute_url(self):
        return 'https://twitter.com/%s/status/%s' % (self.user, self.tweet_id)

    def user_url(self):
        return 'https://twitter.com/%s' % (self.user,)

    def retweeted_user_url(self):
        if self.retweeted_username:
            return 'https://twitter.com/%s' % (self.retweeted_username,)
        else:
            return None


@python_2_unicode_compatible
class Photo(models.Model):
    tweet = models.ForeignKey(Tweet, related_name='photos')
    photo_id = models.BigIntegerField('Photo ID')
    text = models.CharField(max_length=250)
    text_index = models.PositiveIntegerField(db_index=True)
    url = models.URLField('URL')
    media_url = models.URLField('Media URL')
    large_width = models.PositiveIntegerField()
    large_height = models.PositiveIntegerField()

    class Meta:
        ordering = ('tweet', 'text_index')
        unique_together = (
            ('tweet', 'photo_id'),
        )

    def __str__(self):
        return self.text

    def get_absolute_url(self):
        return self.url

    def large_url(self):
        return '%s:large' % (self.media_url,)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
        ('latest_tweets', '0007_tweet_html'),
    ]

    operations = [
        migrations.CreateModel(
            name='Photo',
            fields=[
                ('id', models.AutoField(verbose_name='ID', auto_created=True, primary_key=True, serialize=False)),
                ('photo_id', models.BigIntegerField(verbose_name='Photo ID', unique=True)),
                ('text', models.CharField(max_length=250)),
                ('text_index', models.PositiveIntegerField(db_index=True)),
                ('url', models.URLField(verbose_name='URL')),
                ('media_url', models.URLField(verbose_name='Media URL')),
                ('large_width', models.PositiveIntegerField()),
                ('large_height', models.PositiveIntegerField()),
                ('tweet', models.ForeignKey(related_name='photos', to='latest_tweets.Tweet')),
            ],
            options={
                'ordering': ('tweet', 'text_index'),
            },
        ),
    ]



from django.contrib import admin

from .models import Photo, Tweet


class PhotoInline(admin.StackedInline):
    model = Photo
    readonly_fields = ('photo_id', 'url', 'media_url', 'large_width', 'large_height')
    fieldsets = (
        (None, {
            'fields': readonly_fields,
        }),
    )

    def has_add_permission(self, request):
        return False

    def has_delete_permission(self, request, obj=None):
        return False


@admin.register(Tweet)
class TweetAdmin(admin.ModelAdmin):
    list_display = ('user', 'text', 'created')
    list_filter = ('created', 'user')
    date_hierarchy = 'created'
    readonly_fields = (
        'tweet_id', 'user', 'name', 'text', 'tweet_html', 'favorite_count', 'retweet_count',
        'retweeted_username', 'retweeted_name', 'retweeted_tweet_id', 'is_reply', 'created')
    fieldsets = (
        (None, {
            'fields': readonly_fields,
        }),
    )
    inlines = [PhotoInline]

    def has_add_permission(self, request):
        return False

    def tweet_html(self, obj):
        return obj.html
    tweet_html.short_description = 'HTML'
    tweet_html.allow_tags = True



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='Tweet',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, primary_key=True, auto_created=True)),
                ('tweet_id', models.BigIntegerField(unique=True)),
                ('user', models.CharField(max_length=20, db_index=True)),
                ('text', models.CharField(max_length=250)),
                ('created', models.DateTimeField(db_index=True)),
            ],
            options={
                'ordering': ('-created',),
            },
            bases=(models.Model,),
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
        ('latest_tweets', '0008_photo'),
    ]

    operations = [
        migrations.AddField(
            model_name='tweet',
            name='favorite_count',
            field=models.PositiveIntegerField(default=0),
        ),
        migrations.AddField(
            model_name='tweet',
            name='retweet_count',
            field=models.PositiveIntegerField(default=0),
        ),
    ]



from django import template
from latest_tweets.models import Tweet

register = template.Library()


@register.assignment_tag
def get_latest_tweets(*args, **kwargs):
    limit = kwargs.pop('limit', None)
    include_replies = kwargs.pop('include_replies', False)
    tweets = Tweet.objects.all()

    #  By default we exclude replies
    if not include_replies:
        tweets = tweets.exclude(is_reply=True)

    if args:
        tweets = tweets.filter(user__in=args)

    if limit is not None:
        tweets = tweets[:limit]

    return tweets



#!/usr/bin/env python
from codecs import open

from setuptools import find_packages, setup


with open('README.rst', 'r', 'utf-8') as f:
    readme = f.read()


setup(
    name='django-latest-tweets',
    version='0.4',
    description='Latest Tweets for Django',
    long_description=readme,
    url='https://github.com/blancltd/django-latest-tweets',
    maintainer='Blanc Ltd',
    maintainer_email='studio@blanc.ltd.uk',
    platforms=['any'],
    install_requires=[
        'twitter>=1.9.1',
    ],
    packages=find_packages(),
    classifiers=[
        'Environment :: Web Environment',
        'Framework :: Django',
        'License :: OSI Approved :: BSD License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.2',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
    ],
    license='BSD',
)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
        ('latest_tweets', '0005_names'),
    ]

    operations = [
        migrations.AlterField(
            model_name='tweet',
            name='retweeted_tweet_id',
            field=models.BigIntegerField(verbose_name='Retweeted tweet ID', blank=True, null=True),
            preserve_default=True,
        ),
        migrations.AlterField(
            model_name='tweet',
            name='tweet_id',
            field=models.BigIntegerField(verbose_name='Tweet ID', unique=True),
            preserve_default=True,
        ),
    ]



from __future__ import unicode_literals

from datetime import datetime

from django.utils.six.moves import html_parser
from django.utils.timezone import utc

from .models import Photo, Tweet


HASHTAG_HTML = '<a href="https://twitter.com/hashtag/{text}" target="_blank">#{text}</a>'
URL_HTML = '<a href="{expanded_url}" target="_blank">{display_url}</a>'
MENTION_HTML = '<a href="https://twitter.com/{screen_name}" target="_blank">@{screen_name}</a>'
SYMBOL_HTML = '<a href="https://twitter.com/search?q=%24{text}" target="_blank">${text}</a>'


def tweet_html_entities(tweet, **kwargs):
    text = list(tweet)

    for hashtag in kwargs.get('hashtags', []):
        start, end = hashtag['indices']
        text[start] = HASHTAG_HTML.format(**hashtag)
        text[start + 1:end] = [''] * (end - start - 1)

    for url in kwargs.get('urls', []):
        start, end = url['indices']
        text[start] = URL_HTML.format(**url)
        text[start + 1:end] = [''] * (end - start - 1)

    for mention in kwargs.get('user_mentions', []):
        start, end = mention['indices']
        text[start] = MENTION_HTML.format(**mention)
        text[start + 1:end] = [''] * (end - start - 1)

    for symbol in kwargs.get('symbols', []):
        start, end = symbol['indices']
        text[start] = SYMBOL_HTML.format(**symbol)
        text[start + 1:end] = [''] * (end - start - 1)

    for media in kwargs.get('media', []):
        start, end = media['indices']
        text[start] = URL_HTML.format(**media)
        text[start + 1:end] = [''] * (end - start - 1)

    return ''.join(text)


def tweet_photos(obj, media):
    for photo in media:
        # Only photos
        if photo['type'] != 'photo':
            continue

        photo_id = photo['id']
        large = photo['sizes']['large']

        obj, created = Photo.objects.update_or_create(tweet=obj, photo_id=photo_id, defaults={
            'text': photo['display_url'],
            'text_index': photo['indices'][0],
            'url': photo['url'],
            'media_url': photo['media_url_https'],
            'large_width': int(large['w']),
            'large_height': int(large['h']),
        })


def update_tweets(messages, tweet_entities=tweet_html_entities):
    # Need to escape HTML entities
    htmlparser = html_parser.HTMLParser()
    unescape = htmlparser.unescape

    obj_list = []

    for i in messages:
        tweet_id = i['id']
        tweet_username = i['user']['screen_name']
        tweet_name = i['user']['name']
        tweet_created = datetime.strptime(
            i['created_at'], '%a %b %d %H:%M:%S +0000 %Y'
        ).replace(tzinfo=utc)
        tweet_is_reply = i['in_reply_to_screen_name'] is not None

        if 'retweeted_status' in i:
            retweeted_username = i['retweeted_status']['user']['screen_name']
            retweeted_name = i['retweeted_status']['user']['name']
            retweeted_tweet_id = i['retweeted_status']['id']
            tweet_text = i['retweeted_status']['text']
            tweet_html = tweet_entities(tweet_text, **i['retweeted_status']['entities'])
            favorite_count = i['retweeted_status']['favorite_count']
            retweet_count = i['retweeted_status']['retweet_count']
        else:
            retweeted_username = ''
            retweeted_name = ''
            retweeted_tweet_id = None
            tweet_text = i['text']
            tweet_html = tweet_entities(tweet_text, **i['entities'])
            favorite_count = i['favorite_count']
            retweet_count = i['retweet_count']

        tweet_text = unescape(tweet_text)

        obj, created = Tweet.objects.update_or_create(tweet_id=tweet_id, defaults={
            'user': tweet_username,
            'name': tweet_name,
            'text': tweet_text,
            'html': tweet_html,
            'favorite_count': favorite_count,
            'retweet_count': retweet_count,
            'retweeted_username': retweeted_username,
            'retweeted_name': retweeted_name,
            'retweeted_tweet_id': retweeted_tweet_id,
            'is_reply': tweet_is_reply,
            'created': tweet_created,
        })

        # Add any photos
        tweet_photos(obj=obj, media=i['entities'].get('media', []))

        obj_list.append(obj)

    return obj_list
